@@ -18,7 +18,8 @@
 
 struct ai_choice;
 
-Unit_Type_id ai_choose_defender_versus(struct city *pcity, Unit_Type_id v);
+struct unit_type *ai_choose_defender_versus(struct city *pcity,
+					    const struct unit_type *versus);
 void military_advisor_choose_tech(struct player *pplayer,
 				  struct ai_choice *choice);
 void  military_advisor_choose_build(struct player *pplayer, struct city *pcity,
@@ -27,7 +28,7 @@ void assess_danger_player(struct player *pplayer);
 int assess_defense_quadratic(struct city *pcity);
 int assess_defense_unit(struct city *pcity, struct unit *punit, bool igwall);
 int assess_defense(struct city *pcity);
-int ai_unit_defence_desirability(Unit_Type_id i);
-int ai_unit_attack_desirability(Unit_Type_id i);
+int ai_unit_defence_desirability(const struct unit_type *punittype);
+int ai_unit_attack_desirability(const struct unit_type *punittype);
 
 #endif  /* FC__ADVMILITARY_H */
