@@ -196,7 +196,8 @@ bool client_start_server(void)
 # ifdef WIN32_NATIVE
   STARTUPINFO si;
   PROCESS_INFORMATION pi;
-  
+
+  char savesdir[MAX_LEN_PATH];
   char options[512];
   char cmdline1[512];
   char cmdline2[512];
@@ -311,8 +312,10 @@ bool client_start_server(void)
 		scriptfile);
   }
 
-  my_snprintf(options, sizeof(options), "-p %d -q 1 -e%s%s",
-	      internal_server_port, logcmdline, scriptcmdline);
+  interpret_tilde(savesdir, sizeof(savesdir), "~/.freeciv/saves");
+
+  my_snprintf(options, sizeof(options), "-p %d -q 1 -e%s%s --saves \"%s\"",
+	      internal_server_port, logcmdline, scriptcmdline, savesdir);
   my_snprintf(cmdline1, sizeof(cmdline1), "./ser %s", options);
   my_snprintf(cmdline2, sizeof(cmdline2), "./server/civserver %s", options);
   my_snprintf(cmdline3, sizeof(cmdline3), "civserver %s", options);
@@ -464,13 +467,14 @@ void handle_single_want_hack_reply(bool you_have_hack)
   }
 
   if (you_have_hack) {
-    append_output_window(_("We have control of the server "
-                         "(command access level hack)"));
+    append_output_window(_("Established control over the server. "
+                           "You have command access level 'hack'."));
     client_has_hack = TRUE;
   } else if (is_server_running()) {
     /* only output this if we started the server and we NEED hack */
-    append_output_window(_("We can't take control of server, "
-                         "attempting to kill it."));
+    append_output_window(_("Failed to obtain the required access "
+                           "level to take control of the server. "
+                           "The server will now be shutdown."));
     client_kill_server(TRUE);
   }
 }
@@ -483,7 +487,6 @@ void send_start_saved_game(void)
   char buf[MAX_LEN_MSG];
 
   send_chat("/set timeout 0");
-  send_chat("/set autotoggle 1");
   my_snprintf(buf, sizeof(buf), "/take \"%s\" \"%s\"",
       	      user_name, leader_name);
   send_chat(buf);
