@@ -18,6 +18,7 @@
 #include <assert.h>
 
 #include "fcintl.h"
+#include "hash.h"
 #include "log.h"
 #include "mem.h"
 #include "timing.h"
@@ -309,6 +310,20 @@ static void current_focus_append(struct unit *punit)
   punit->focus_status = FOCUS_AVAIL;
   refresh_unit_mapcanvas(punit, punit->tile, TRUE, FALSE);
 
+  if (unit_selection_clears_orders) {
+    clear_unit_orders(punit);
+  }
+}
+
+/**************************************************************************
+  Clear all orders for the given unit.
+**************************************************************************/
+void clear_unit_orders(struct unit *punit)
+{
+  if (!punit) {
+    return;
+  }
+
   if (unit_has_orders(punit)) {
     /* Clear the focus unit's orders. */
     request_orders_cleared(punit);
@@ -931,6 +946,10 @@ void control_mouse_cursor(struct tile *ptile)
   struct city *pcity = NULL;
   struct unit_list *active_units = get_units_in_focus();
 
+  if (!enable_cursor_changes) {
+    return;
+  }
+
   if (C_S_RUNNING != client_state()) {
     action_state = CURSOR_ACTION_DEFAULT;
     return;
@@ -1192,22 +1211,75 @@ void request_unit_wakeup(struct unit *punit)
 }
 
 /****************************************************************************
-  Select all units of the same type as the given unit.
+  Select all units based on the given list of units and the selection modes.
 ****************************************************************************/
-void request_unit_select_same_type(struct unit_list *punits)
+void request_unit_select(struct unit_list *punits,
+                         enum unit_select_type_mode seltype,
+                         enum unit_select_location_mode selloc)
 {
-  if (can_client_change_view()) {
-    unit_list_iterate(punits, punit) {
-      unit_list_iterate(unit_owner(punit)->units, punit2) {
-	if (unit_type(punit2) == unit_type(punit)
-	    && !unit_list_search(punits, punit2)
-	    && punit2->activity == ACTIVITY_IDLE
-	    && !unit_has_orders(punit2)) {
-	  add_unit_focus(punit2);
-	}
+  const struct player *pplayer;
+  struct unit *punit_first;
+  struct hash_table *tile_table, *type_table, *cont_table;
+
+  if (!can_client_change_view() || !punits
+      || unit_list_size(punits) < 1) {
+    return;
+  }
+
+  punit_first = unit_list_get(punits, 0);
+
+  if (seltype == SELTYPE_SINGLE) {
+    set_unit_focus(punit_first);
+    return;
+  }
+
+  pplayer = unit_owner(punit_first);
+  tile_table = hash_new(hash_fval_keyval, hash_fcmp_keyval);
+  type_table = hash_new(hash_fval_keyval, hash_fcmp_keyval);
+  cont_table = hash_new(hash_fval_int, hash_fcmp_int);
+
+  unit_list_iterate(punits, punit) {
+    if (seltype == SELTYPE_SAME) {
+      hash_insert(type_table, unit_type(punit), NULL);
+    }
+
+    if (selloc == SELLOC_TILE) {
+      hash_insert(tile_table, punit->tile, NULL);
+    } else if (selloc == SELLOC_CONT) {
+      hash_insert(cont_table, &punit->tile->continent, NULL);
+    }
+  } unit_list_iterate_end;
+
+  if (selloc == SELLOC_TILE) {
+    hash_keys_iterate(tile_table, key) {
+      const struct tile *ptile = key;
+      unit_list_iterate(ptile->units, punit) {
+        if (unit_owner(punit) != pplayer) {
+          continue;
+        }
+        if (seltype == SELTYPE_SAME
+            && !hash_key_exists(type_table, unit_type(punit))) {
+          continue;
+        }
+        add_unit_focus(punit);
       } unit_list_iterate_end;
+    } hash_keys_iterate_end;
+  } else {
+    unit_list_iterate(pplayer->units, punit) {
+      if ((seltype == SELTYPE_SAME
+           && !hash_key_exists(type_table, unit_type(punit)))
+          || (selloc == SELLOC_CONT
+              && !hash_key_exists(cont_table, &punit->tile->continent))) {
+        continue;
+      }
+
+      add_unit_focus(punit);
     } unit_list_iterate_end;
   }
+
+  hash_free(tile_table);
+  hash_free(type_table);
+  hash_free(cont_table);
 }
 
 /**************************************************************************
@@ -1622,6 +1694,32 @@ void request_toggle_city_productions(void)
   update_map_canvas_visible();
 }
 
+/**************************************************************************
+ Toggle display of city buycost
+**************************************************************************/
+void request_toggle_city_buycost(void)
+{
+  if (!can_client_change_view()) {
+    return;
+  }
+
+  draw_city_buycost ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of city traderoutes
+**************************************************************************/
+void request_toggle_city_traderoutes(void)
+{
+  if (!can_client_change_view()) {
+    return;
+  }
+
+  draw_city_traderoutes ^= 1;
+  update_map_canvas_visible();
+}
+
 /**************************************************************************
  Toggle display of terrain
 **************************************************************************/
@@ -1811,6 +1909,7 @@ void request_unit_move_done(void)
 {
   if (get_num_units_in_focus() > 0) {
     unit_list_iterate(get_units_in_focus(), punit) {
+      clear_unit_orders(punit);
       punit->focus_status = FOCUS_DONE;
     } unit_list_iterate_end;
     advance_unit_focus();
@@ -2683,6 +2782,23 @@ void key_city_productions_toggle(void)
   request_toggle_city_productions();
 }
 
+/**************************************************************************
+  Toggles the showing of the buy cost of the current production in the
+  city descriptions.
+**************************************************************************/
+void key_city_buycost_toggle(void)
+{
+  request_toggle_city_buycost();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_city_traderoutes_toggle(void)
+{
+  request_toggle_city_traderoutes();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
