@@ -96,7 +96,25 @@ void request_unit_return(struct unit *punit);
 void request_unit_upgrade(struct unit *punit);
 void request_units_wait(struct unit_list *punits);
 void request_unit_wakeup(struct unit *punit);
-void request_unit_select_same_type(struct unit_list *punits);
+
+enum unit_select_type_mode {
+  SELTYPE_SINGLE = 0,
+  SELTYPE_SAME,
+  SELTYPE_ALL,
+  NUM_SELTYPES
+};
+
+enum unit_select_location_mode {
+  SELLOC_TILE = 0,
+  SELLOC_CONT, /* Continent. */
+  SELLOC_ALL,
+  NUM_SELLOCS
+};
+
+void request_unit_select(struct unit_list *punits,
+                         enum unit_select_type_mode seltype,
+                         enum unit_select_location_mode selloc);
+
 void request_diplomat_action(enum diplomat_actions action, int dipl_id,
 			     int target_id, int value);
 void request_diplomat_answer(enum diplomat_actions action, int dipl_id,
@@ -107,6 +125,8 @@ void request_toggle_map_borders(void);
 void request_toggle_city_names(void);
 void request_toggle_city_growth(void);
 void request_toggle_city_productions(void);
+void request_toggle_city_buycost(void);
+void request_toggle_city_traderoutes(void);
 void request_toggle_terrain(void);
 void request_toggle_coastline(void);
 void request_toggle_roads_rails(void);
@@ -121,6 +141,7 @@ void request_toggle_focus_unit(void);
 void request_toggle_fog_of_war(void);
 
 void wakeup_sentried_units(struct tile *ptile);
+void clear_unit_orders(struct unit *punit);
 
 bool unit_is_in_focus(const struct unit *punit);
 struct unit *get_focus_unit_on_tile(const struct tile *ptile);
@@ -151,6 +172,8 @@ void key_center_capital(void);
 void key_city_names_toggle(void);
 void key_city_growth_toggle(void);
 void key_city_productions_toggle(void);
+void key_city_buycost_toggle(void);
+void key_city_traderoutes_toggle(void);
 void key_terrain_toggle(void);
 void key_coastline_toggle(void);
 void key_roads_rails_toggle(void);
