@@ -58,7 +58,7 @@
 #define CMA_CUSTOM	(10001)
 
 enum city_operation_type {
-  CO_CHANGE, CO_LAST, CO_NEXT, CO_FIRST, CO_NONE
+  CO_CHANGE, CO_LAST, CO_NEXT, CO_FIRST, CO_NEXT_TO_LAST, CO_NONE
 };
 
 /******************************************************************/
@@ -72,12 +72,15 @@ static void city_command_callback(struct gui_dialog *dlg, int response,
                                   gpointer data);
 
 static void city_selection_changed_callback(GtkTreeSelection *selection);
+static void city_clear_worklist_callback(GtkMenuItem *item, gpointer data);
+static void update_total_buy_cost(void);
 
 static void create_select_menu(GtkWidget *item);
 static void create_change_menu(GtkWidget *item);
 static void create_last_menu(GtkWidget *item);
 static void create_first_menu(GtkWidget *item);
 static void create_next_menu(GtkWidget *item);
+static void create_next_to_last_menu(GtkWidget *item);
 
 static struct gui_dialog *city_dialog_shell = NULL;
 
@@ -94,16 +97,19 @@ static void popup_change_menu(GtkMenuShell *menu, gpointer data);
 static void popup_last_menu(GtkMenuShell *menu, gpointer data);
 static void popup_first_menu(GtkMenuShell *menu, gpointer data);
 static void popup_next_menu(GtkMenuShell *menu, gpointer data);
+static void popup_next_to_last_menu(GtkMenuShell *menu, gpointer data);
 
-static GtkWidget *city_center_command, *city_popup_command, *city_buy_command;
+static GtkWidget *city_center_command;
+static GtkWidget *city_popup_command;
+static GtkWidget *city_buy_command;
 static GtkWidget *city_production_command;
+static GtkWidget *city_governor_command;
 static GtkWidget *city_total_buy_cost_label;
 
 
 static GtkWidget *change_improvements_item;
 static GtkWidget *change_units_item;
 static GtkWidget *change_wonders_item;
-static GtkWidget *change_cma_item;
 
 static GtkWidget *last_improvements_item;
 static GtkWidget *last_units_item;
@@ -117,6 +123,9 @@ static GtkWidget *next_improvements_item;
 static GtkWidget *next_units_item;
 static GtkWidget *next_wonders_item;
 
+static GtkWidget *next_to_last_improvements_item;
+static GtkWidget *next_to_last_units_item;
+static GtkWidget *next_to_last_wonders_item;
 
 static GtkWidget *select_island_item;
 
@@ -393,6 +402,32 @@ static void worklist_next_impr_or_unit_iterate(GtkTreeModel *model,
   /* perhaps should warn the user if not successful? */
 }
 
+/**************************************************************************
+  Called by select_impr_or_unit_callback for each city that is selected in
+  the city list dialog to have an object added before the last position in
+  the worklist.
+**************************************************************************/
+static void worklist_next_to_last_impr_or_unit_iterate(GtkTreeModel *model,
+                                                       GtkTreePath *path,
+                                                       GtkTreeIter *it,
+                                                       gpointer data)
+{
+  struct city_production target;
+  struct city *pcity;
+  gint id, n;
+
+  target = cid_decode(GPOINTER_TO_INT(data));
+
+  gtk_tree_model_get(model, it, 1, &id, -1);
+  pcity = game_find_city_by_number(id);
+  if (!pcity) {
+    return;
+  }
+
+  n = worklist_length(&pcity->worklist);
+  city_queue_insert(pcity, n, target);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -422,6 +457,7 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
       }
     }
   } else {
+    GtkTreeSelectionForeachFunc foreach_func;
     connection_do_buffer(&aconnection);
     switch (city_operation) {
     case CO_LAST:
@@ -444,6 +480,11 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
 					  worklist_next_impr_or_unit_iterate,
 					  GINT_TO_POINTER(cid_encode(target)));
       break;
+    case CO_NEXT_TO_LAST:
+      foreach_func = worklist_next_to_last_impr_or_unit_iterate;
+      gtk_tree_selection_selected_foreach(city_selection, foreach_func,
+                                          GINT_TO_POINTER(cid_encode(target)));
+      break;
     default:
       assert(FALSE); /* should never get here. */
     }
@@ -542,6 +583,11 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
   struct cm_parameter parameter;
   GtkWidget *w;
 
+  w = gtk_menu_item_get_submenu(parent_item);
+  if (w != NULL && GTK_WIDGET_VISIBLE(w)) {
+    return;
+  }
+
   gtk_menu_item_remove_submenu(parent_item);
   if (!can_client_issue_orders()) {
     return;
@@ -629,20 +675,151 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
   gtk_widget_show_all(menu);
 }
 
+/**************************************************************************
+  Helper function to append a worklist to the current work list of one city
+  in the city report. This function is called over all selected rows in the
+  list view.
+**************************************************************************/
+static void append_worklist_foreach(GtkTreeModel *model, GtkTreePath *path,
+                                    GtkTreeIter *it, gpointer data)
+{
+  struct worklist *pwl;
+  struct city *pcity;
+
+  pwl = data;
+  g_return_if_fail(pwl != NULL);
+  g_return_if_fail(pwl->is_valid);
+
+  gtk_tree_model_get(model, it, 0, &pcity, -1);
+  if (!pcity || !game_find_city_by_number(pcity->id)) {
+    return;
+  }
+  city_queue_insert_worklist(pcity, -1, pwl);
+}
+
+/**************************************************************************
+  Menu item callback to append the global worklist associated with this
+  item to the worklists of all selected cities. The worklist pointer is
+  passed in 'data'.
+**************************************************************************/
+static void append_worklist_callback(GtkMenuItem *menuitem, gpointer data)
+{
+  g_return_if_fail(city_selection != NULL);
+  gtk_tree_selection_selected_foreach(city_selection,
+                                      append_worklist_foreach, data);
+}
+
+/**************************************************************************
+  Helper function to set a worklist for one city in the city report. This
+  function is called over all selected rows in the list view.
+**************************************************************************/
+static void set_worklist_foreach(GtkTreeModel *model, GtkTreePath *path,
+                                 GtkTreeIter *it, gpointer data)
+{
+  struct worklist *pwl;
+  struct city *pcity;
+
+  pwl = data;
+  g_return_if_fail(pwl != NULL);
+  g_return_if_fail(pwl->is_valid);
+
+  gtk_tree_model_get(model, it, 0, &pcity, -1);
+  if (!pcity || !game_find_city_by_number(pcity->id)) {
+    return;
+  }
+  city_set_queue(pcity, pwl);
+}
+
+/**************************************************************************
+  Menu item callback to set a city's worklist to the global worklist
+  associated with this menu item. The worklist pointer is passed in 'data'.
+**************************************************************************/
+static void set_worklist_callback(GtkMenuItem *menuitem, gpointer data)
+{
+  g_return_if_fail(city_selection != NULL);
+  gtk_tree_selection_selected_foreach(city_selection, set_worklist_foreach,
+                                      data);
+}
+
+/**************************************************************************
+  Empty and refill the submenu of the menu item passed as 'data'. The menu
+  will be filled with menu items corresponding to the global worklists.
+**************************************************************************/
+static void production_menu_shown(GtkWidget *widget, gpointer data)
+{
+  struct worklist *worklists;
+  struct player *pplayer;
+  GtkWidget *menu, *item;
+  GtkMenuItem *parent_item;
+  GCallback callback;
+  int i, count = 0;
+
+  parent_item = data;
+  g_return_if_fail(parent_item != NULL);
+  g_return_if_fail(GTK_IS_MENU_ITEM(parent_item));
+
+  callback = g_object_get_data(G_OBJECT(parent_item), "item_callback");
+  g_return_if_fail(callback != NULL);
+
+  menu = gtk_menu_item_get_submenu(parent_item);
+  if (menu != NULL && GTK_WIDGET_VISIBLE(menu)) {
+    gtk_menu_shell_deactivate(GTK_MENU_SHELL(menu));
+  }
+
+  if (menu == NULL) {
+    menu = gtk_menu_new();
+    gtk_menu_item_set_submenu(parent_item, menu);
+  }
+
+  pplayer = game.player_ptr;
+  if (!pplayer || !can_client_issue_orders()) {
+    return;
+  }
+
+  gtk_container_forall(GTK_CONTAINER(menu),
+                       (GtkCallback) gtk_widget_destroy, NULL);
+
+  worklists = pplayer->worklists;
+  for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
+    if (!worklists[i].is_valid) {
+      continue;
+    }
+    item = gtk_menu_item_new_with_label(worklists[i].name);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    g_signal_connect(item, "activate", callback, &worklists[i]);
+    count++;
+  }
+
+  if (count == 0) {
+    item = gtk_menu_item_new_with_label(_("(no worklists defined)"));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  }
+
+  gtk_widget_show_all(menu);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
 static void city_report_update_views(void)
 {
   struct city_report_spec *spec;
-  int i;
+  GtkTreeView *view;
+  GtkTreeViewColumn *col;
+  GList *columns, *p;
 
-  for (i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
-    GtkTreeViewColumn *col;
+  view = GTK_TREE_VIEW(city_view);
+  g_return_if_fail(view != NULL);
 
-    col = gtk_tree_view_get_column(GTK_TREE_VIEW(city_view), i);
+  columns = gtk_tree_view_get_columns(view);
+
+  for (p = columns; p != NULL; p = p->next) {
+    col = p->data;
+    spec = g_object_get_data(G_OBJECT(col), "city_report_spec");
     gtk_tree_view_column_set_visible(col, spec->show);
   }
+
+  g_list_free(columns);
 }
 
 /****************************************************************
@@ -708,10 +885,39 @@ static GtkWidget *create_city_report_menubar(void)
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   create_next_menu(item);
 
+  item = gtk_menu_item_new_with_mnemonic(_("Add _2nd Last"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  create_next_to_last_menu(item);
+
   item = gtk_menu_item_new_with_mnemonic(_("Add _Last"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   create_last_menu(item);
 
+  item = gtk_separator_menu_item_new();
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+
+  item = gtk_menu_item_new_with_label(_("Set Worklist"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  g_object_set_data(G_OBJECT(item), "item_callback",
+                    set_worklist_callback);
+  g_signal_connect(menu, "show", G_CALLBACK(production_menu_shown), item);
+
+  item = gtk_menu_item_new_with_label(_("Append Worklist"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  g_object_set_data(G_OBJECT(item), "item_callback",
+                    append_worklist_callback);
+  g_signal_connect(menu, "show", G_CALLBACK(production_menu_shown), item);
+
+  item = gtk_menu_item_new_with_mnemonic(_("Clear _Worklist"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(city_clear_worklist_callback), NULL);
+
+  item = gtk_menu_item_new_with_mnemonic(_("Gover_nor"));
+  city_governor_command = item;
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  append_cma_to_menu_item(GTK_MENU_ITEM(item), TRUE);
+
   item = gtk_menu_item_new_with_mnemonic(_("_Select"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
   create_select_menu(item);
@@ -864,6 +1070,8 @@ static void create_city_report_dialog(bool make_modal)
     col = gtk_tree_view_column_new_with_attributes(titles[i], renderer,NULL);
     gtk_tree_view_column_set_visible(col, spec->show);
     gtk_tree_view_column_set_sort_column_id(col, i);
+    gtk_tree_view_column_set_reorderable(col, TRUE);
+    g_object_set_data(G_OBJECT(col), "city_report_spec", spec);
     gtk_tree_view_append_column(GTK_TREE_VIEW(city_view), col);
     gtk_tree_view_column_set_cell_data_func(col, renderer,
       cityrep_cell_data_func, GINT_TO_POINTER(i), NULL);
@@ -1173,6 +1381,10 @@ void city_report_dialog_update(void)
 
     city_selection_changed_callback(city_selection);
 
+    if (GTK_WIDGET_SENSITIVE(city_governor_command)) {
+      append_cma_to_menu_item(GTK_MENU_ITEM(city_governor_command), TRUE);
+    }
+
     select_menu_cached = FALSE;
   }
 }
@@ -1203,8 +1415,8 @@ void city_report_dialog_update_city(struct city *pcity)
     /* update. */
     if (found) {
       update_row(TREE_ITER_PTR(it), pcity);
-
       select_menu_cached = FALSE;
+      update_total_buy_cost();
     } else {
       city_report_dialog_update();
     }
@@ -1228,8 +1440,6 @@ static void create_change_menu(GtkWidget *item)
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), change_improvements_item);
   change_wonders_item = gtk_menu_item_new_with_label(_("Wonders"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), change_wonders_item);
-  change_cma_item = gtk_menu_item_new_with_label(_("Citizen Governor"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), change_cma_item);
 }
 
 /****************************************************************
@@ -1289,6 +1499,30 @@ static void create_next_menu(GtkWidget *item)
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), next_wonders_item);
 }
 
+/**************************************************************************
+  Append the "next to last" submenu to the given menu item.
+**************************************************************************/
+static void create_next_to_last_menu(GtkWidget *parent_item)
+{
+  GtkWidget *menu, *item;
+
+  menu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(parent_item), menu);
+  g_signal_connect(menu, "show",
+                   G_CALLBACK(popup_next_to_last_menu), NULL);
+
+  item = gtk_menu_item_new_with_label(_("Units"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  next_to_last_units_item = item;
+
+  item = gtk_menu_item_new_with_label(_("Improvements"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  next_to_last_improvements_item = item;
+
+  item = gtk_menu_item_new_with_label(_("Wonders"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  next_to_last_wonders_item = item;
+}
 
 
 /****************************************************************
@@ -1312,7 +1546,6 @@ static void popup_change_menu(GtkMenuShell *menu, gpointer data)
 				  FALSE, TRUE, CO_CHANGE,
 				  city_can_build_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
-  append_cma_to_menu_item(GTK_MENU_ITEM(change_cma_item), TRUE);
 }
 
 /****************************************************************
@@ -1384,7 +1617,36 @@ static void popup_next_menu(GtkMenuShell *menu, gpointer data)
 				  G_CALLBACK(select_impr_or_unit_callback), n);
 }
 
+/**************************************************************************
+  Re-create the submenus in the next-to-last production change menu.
+**************************************************************************/
+static void popup_next_to_last_menu(GtkMenuShell *menu, gpointer data)
+{
+  GtkWidget *item;
+  GCallback callback;
+  int n;
 
+  g_return_if_fail(city_selection != NULL);
+
+  n = gtk_tree_selection_count_selected_rows(city_selection);
+  callback = G_CALLBACK(select_impr_or_unit_callback);
+
+  item = next_to_last_improvements_item;
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(item),
+                                   FALSE, FALSE, CO_NEXT_TO_LAST,
+                                   city_can_build_impr_or_unit,
+                                   callback, n);
+  item = next_to_last_units_item;
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(item),
+                                   TRUE, FALSE, CO_NEXT_TO_LAST,
+                                   city_can_build_impr_or_unit,
+                                   callback, n);
+  item = next_to_last_wonders_item;
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(item),
+                                   FALSE, TRUE, CO_NEXT_TO_LAST,
+                                   city_can_build_impr_or_unit,
+                                   callback, n);
+}
 
 /****************************************************************
 ...
@@ -1576,57 +1838,107 @@ static void popup_select_menu(GtkMenuShell *menu, gpointer data)
   select_menu_cached = TRUE;
 }
 
-/****************************************************************
-...
-*****************************************************************/
+/***************************************************************************
+  Update the value displayed by the "total buy cost" label in the city
+  report, or make it blank if nothing can be bought.
+***************************************************************************/
+static void update_total_buy_cost(void)
+{
+  GtkWidget *label, *view;
+  GList *rows, *p;
+  GtkTreeModel *model;
+  GtkTreeSelection *sel;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+  gpointer res;
+  struct city *pcity;
+  int total = 0;
+
+  view = city_view;
+  label = city_total_buy_cost_label;
+
+  if (!view || !label) {
+    return;
+  }
+
+  sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
+  rows = gtk_tree_selection_get_selected_rows(sel, &model);
+
+  for (p = rows; p != NULL; p = p->next) {
+    path = p->data;
+    if (gtk_tree_model_get_iter(model, &iter, path)) {
+      gtk_tree_model_get(model, &iter, 0, &res, -1);
+      pcity = res;
+      if (pcity != NULL) {
+        total += city_buy_cost(pcity);
+      }
+    }
+    gtk_tree_path_free(path);
+  }
+  g_list_free(rows);
+
+  if (total > 0) {
+    char buf[128];
+    my_snprintf(buf, sizeof(buf), _("Total Buy Cost: %d"), total);
+    gtk_label_set_text(GTK_LABEL(label), buf);
+  } else {
+    gtk_label_set_text(GTK_LABEL(label), NULL);
+  }
+}
+
+/***************************************************************************
+  Update city report button sensitivity and total buy cost label when the
+  user makes a change in the selection of cities.
+***************************************************************************/
 static void city_selection_changed_callback(GtkTreeSelection *selection)
 {
   int n;
-  int total = 0;
 
   n = gtk_tree_selection_count_selected_rows(selection);
 
-  if (n == 0) {
-    gtk_widget_set_sensitive(city_production_command, FALSE);
-    gtk_widget_set_sensitive(city_center_command, FALSE);
-    gtk_widget_set_sensitive(city_popup_command, FALSE);
-    gtk_widget_set_sensitive(city_buy_command, FALSE);
-  } else {
-    GList *rows, *p;
-    GtkTreeModel *model;
-    GtkTreePath *path;
-    GtkTreeIter iter;
-    gpointer res;
-    struct city *pcity;
+  gtk_widget_set_sensitive(city_production_command,
+                           n > 0 && can_client_issue_orders());
+  gtk_widget_set_sensitive(city_governor_command,
+                           n > 0 && can_client_issue_orders());
+  gtk_widget_set_sensitive(city_center_command, n > 0);
+  gtk_widget_set_sensitive(city_popup_command, n > 0);
+  gtk_widget_set_sensitive(city_buy_command,
+                           n > 0 && can_client_issue_orders());
 
-    gtk_widget_set_sensitive(city_production_command,
-			     can_client_issue_orders());
-    gtk_widget_set_sensitive(city_center_command, TRUE);
-    gtk_widget_set_sensitive(city_popup_command, TRUE);
-    gtk_widget_set_sensitive(city_buy_command, can_client_issue_orders());
+  update_total_buy_cost();
+}
 
-    rows = gtk_tree_selection_get_selected_rows(selection, &model);
-    for (p = rows; p != NULL; p = p->next) {
-      path = p->data;
-      if (gtk_tree_model_get_iter(model, &iter, path)) {
-        gtk_tree_model_get(model, &iter, 0, &res, -1);
-        pcity = res;
-        if (pcity != NULL) {
-          total += city_buy_cost(pcity);
-        }
-      }
-      gtk_tree_path_free(path);
-    }
-    g_list_free(rows);
+/**************************************************************************
+  Clear the worklist in one selected city in the city report.
+**************************************************************************/
+static void clear_worklist_foreach_func(GtkTreeModel *model,
+                                        GtkTreePath *path,
+                                        GtkTreeIter *it,
+                                        gpointer data)
+{
+  struct city *pcity;
+
+  gtk_tree_model_get(model, it, 0, &pcity, -1);
+  if (pcity && game_find_city_by_number(pcity->id)) {
+    struct worklist empty;
+    init_worklist(&empty);
+    city_set_worklist(pcity, &empty);
   }
+}
 
-  if (total > 0) {
-    char buf[64];
-    my_snprintf(buf, sizeof(buf), _("Total Buy Cost: %d"), total);
-    gtk_label_set_text(GTK_LABEL(city_total_buy_cost_label), buf);
-  } else {
-    gtk_label_set_text(GTK_LABEL(city_total_buy_cost_label), NULL);
-  }
+/**************************************************************************
+  Called when the "clear worklist" menu item is activated.
+**************************************************************************/
+static void city_clear_worklist_callback(GtkMenuItem *item, gpointer data)
+{
+  struct connection *pconn = &aconnection;
+
+  g_return_if_fail(city_selection != NULL);
+
+  connection_do_buffer(pconn);
+  gtk_tree_selection_selected_foreach(city_selection,
+                                      clear_worklist_foreach_func, NULL);
+  connection_do_unbuffer(pconn);
 }
 
 /****************************************************************
