@@ -55,6 +55,7 @@
 #include "clinet.h"
 #include "colors.h"
 #include "connectdlg.h"
+#include "connectdlg_common.h"
 #include "control.h"
 #include "cma_fe.h"
 #include "dialogs.h"
@@ -94,13 +95,20 @@ int overview_canvas_store_height = 2 * 50;
 
 bool enable_tabs = TRUE;
 bool better_fog = TRUE;
+bool show_chat_message_time = FALSE;
+bool split_bottom_notebook = FALSE;
+bool new_messages_go_to_top = FALSE;
+bool show_message_window_buttons = TRUE;
+bool metaserver_tab_first = FALSE;
+bool allied_chat_only = FALSE;
 
 GtkWidget *toplevel;
 GdkWindow *root_window;
 GtkWidget *toplevel_tabs;
 GtkWidget *top_vbox;
-GtkWidget *top_notebook, *bottom_notebook;
+GtkWidget *top_notebook, *bottom_notebook, *right_notebook;
 GtkWidget *map_widget;
+static GtkWidget *bottom_hpaned;
 
 int city_names_font_size = 0, city_productions_font_size = 0;
 PangoFontDescription *main_font;
@@ -114,6 +122,7 @@ GdkGC *fill_tile_gc;
 GdkGC *thin_line_gc;
 GdkGC *thick_line_gc;
 GdkGC *border_line_gc;
+GdkGC *selection_gc;
 GdkPixmap *gray50, *gray25, *black50;
 GdkPixmap *mask_bitmap;
 
@@ -145,6 +154,9 @@ GtkWidget *government_ebox;
 const char * const gui_character_encoding = "UTF-8";
 const bool gui_use_transliteration = FALSE;
 
+static void split_bottom_notebook_callback(struct client_option *op);
+static void allied_chat_only_callback(struct client_option *op);
+
 client_option gui_options[] = {
   /* This option is the same as the one in gui-gtk */
   GEN_BOOL_OPTION(map_scrollbars, N_("Show Map Scrollbars"),
@@ -178,7 +190,55 @@ client_option gui_options[] = {
 		     N_("If this is enabled then a better method is used "
 			"for drawing fog-of-war.  It is not any slower but "
 			"will consume about twice as much memory."),
-		     COC_GRAPHICS, mapview_redraw_callback)
+		     COC_GRAPHICS, mapview_redraw_callback),
+  GEN_BOOL_OPTION(show_chat_message_time,
+                  N_("Show time for each chat message"),
+                  N_("If this option is enabled then all chat messages "
+                     "will be prefixed by a time string of the form "
+                     "[hour:minute:second]."),
+                  COC_INTERFACE),
+  GEN_BOOL_OPTION_CB(split_bottom_notebook,
+                     N_("Split bottom notebook area"),
+                     N_("Enabling this option will split the bottom "
+                        "notebook into a left and right notebook so that "
+                        "two tabs may be viewed at once."),
+                     COC_INTERFACE, split_bottom_notebook_callback),
+  GEN_BOOL_OPTION(new_messages_go_to_top,
+                  N_("New message events go to top of list"),
+                  N_("If this option is enabled, new events in the "
+                     "message window will appear at the top of the list, "
+                     "rather than being appended at the bottom."),
+                  COC_INTERFACE),
+  GEN_BOOL_OPTION(show_message_window_buttons,
+                  N_("Show extra message window buttons"),
+                  N_("If this option is enabled, there will be two "
+                     "buttons displayed in the message window for "
+                     "inspecting a city and going to a location. If this "
+                     "option is disabled, these buttons will not appear "
+                     "(you can still double-click with the left mouse "
+                     "button or right-click on a row to inspect or goto "
+                     "respectively). This option will only take effect "
+                     "once the message window is closed and reopened."),
+                  COC_INTERFACE),
+  GEN_BOOL_OPTION(metaserver_tab_first,
+                  N_("Metaserver tab first in network page"),
+                  N_("If this option is enabled, the metaserver tab will "
+                     "be the first notebook tab in the network page. This "
+                     "option requires a restart in order to take effect."),
+                  COC_NETWORK),
+  GEN_BOOL_OPTION_CB(allied_chat_only,
+                     N_("Plain chat messages are sent to allies only"),
+                     N_("If this option is enabled, then plain messages "
+                        "typed into the chat entry while the game is "
+                        "running will only be sent to your allies. "
+                        "Otherwise plain messages will be sent as "
+                        "public chat messages. To send a public chat "
+                        "message with this option enabled, prefix the "
+                        "message with a single colon ':'. This option "
+                        "can also be set using a toggle button beside "
+                        "the chat entry (only visible in multiplayer "
+                        "games)."),
+                     COC_NETWORK, allied_chat_only_callback)
 };
 const int num_gui_options = ARRAY_SIZE(gui_options);
 
@@ -196,6 +256,7 @@ static int unit_ids[MAX_NUM_UNITS_BELOW];  /* ids of the units icons in
 GtkTextView *main_message_area;
 GtkTextBuffer *message_buffer;
 static GtkWidget *inputline;
+static GtkWidget *allied_chat_toggle_button;
 
 static enum Display_color_type display_color_type;  /* practically unused */
 static gint timer_id;                               /*       ditto        */
@@ -224,6 +285,8 @@ static gboolean select_unit_pixmap_callback(GtkWidget *w, GdkEvent *ev,
 					    gpointer data);
 static gboolean quit_dialog_callback(void);
 
+static void allied_chat_button_toggled(GtkToggleButton *button,
+                                       gpointer user_data);
 
 /****************************************************************************
   Called by the tileset code to set the font size that should be used to
@@ -397,6 +460,48 @@ static gboolean toplevel_handler(GtkWidget *w, GdkEventKey *ev, gpointer data)
   return FALSE;
 }
 
+
+/**************************************************************************
+  Idle callback for scrolling down the chatline.
+  NB: Only to be used by queue_chatline_scroll_to_bottom().
+**************************************************************************/
+static gboolean chatline_scroll_callback(gpointer data)
+{
+  guint *pid = data;
+  chatline_scroll_to_bottom();
+  *pid = 0;
+  return FALSE; /* Remove this idle function. */
+}
+
+/**************************************************************************
+  Adds an idle callback to scroll the chatline to the bottom.
+**************************************************************************/
+static void queue_chatline_scroll_to_bottom(void)
+{
+  static guint id = 0;
+  if (id == 0) {
+    id = g_idle_add(chatline_scroll_callback, &id);
+  }
+}
+
+/**************************************************************************
+  When the chatline text view is resized, scroll it to the bottom. This
+  prevents users from accidentally missing messages when the chatline
+  gets scrolled up a small amount and stops scrolling down automatically.
+**************************************************************************/
+static void main_message_area_size_allocate(GtkWidget *widget,
+                                            GtkAllocation *allocation,
+                                            gpointer data)
+{
+  static int old_width = 0, old_height = 0;
+  if (allocation->width != old_width
+      || allocation->height != old_height) {
+    queue_chatline_scroll_to_bottom();
+    old_width = allocation->width;
+    old_height = allocation->height;
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -561,6 +666,13 @@ static gboolean keyboard_map_canvas(GtkWidget *w, GdkEventKey *ev, gpointer data
     key_cancel_action();
     return TRUE;
 
+  case GDK_b:
+    if (tiles_hilited_cities) {
+      buy_production_in_selected_cities();
+      return TRUE;
+    }
+    break;
+
   default:
     break;
   };
@@ -807,8 +919,6 @@ static void populate_unit_pixmap_table(void)
       
     gtk_table_attach_defaults(GTK_TABLE(table), unit_below_pixmap_button[i],
                               i, i + 1, 1, 2);
-    gtk_widget_set_size_request(unit_below_pixmap[i],
-				tileset_full_tile_width(tileset), tileset_full_tile_height(tileset));
     gtk_pixcomm_clear(GTK_PIXCOMM(unit_below_pixmap[i]));
   }
 
@@ -827,23 +937,25 @@ void reset_unit_table(void)
 {
   int i;
 
-  /* Unreference all of the widgets that we're about to reallocate, thus
-   * avoiding a memory leak. Remove them from the container first, just
-   * to be safe. Note, the widgets are ref'd in
-   * populatate_unit_pixmap_table. */
-  gtk_container_remove(GTK_CONTAINER(unit_pixmap_table),
-		       unit_pixmap_button);
-  gtk_widget_unref(unit_pixmap);
-  gtk_widget_unref(unit_pixmap_button);
-  for (i = 0; i < num_units_below; i++) {
+  if (unit_pixmap_button) {
+    /* Unreference all of the widgets that we're about to reallocate, thus
+     * avoiding a memory leak. Remove them from the container first, just
+     * to be safe. Note, the widgets are ref'd in
+     * populatate_unit_pixmap_table. */
     gtk_container_remove(GTK_CONTAINER(unit_pixmap_table),
-			 unit_below_pixmap_button[i]);
-    gtk_widget_unref(unit_below_pixmap[i]);
-    gtk_widget_unref(unit_below_pixmap_button[i]);
+                         unit_pixmap_button);
+    gtk_widget_unref(unit_pixmap);
+    gtk_widget_unref(unit_pixmap_button);
+    for (i = 0; i < num_units_below; i++) {
+      gtk_container_remove(GTK_CONTAINER(unit_pixmap_table),
+                           unit_below_pixmap_button[i]);
+      gtk_widget_unref(unit_below_pixmap[i]);
+      gtk_widget_unref(unit_below_pixmap_button[i]);
+    }
+    gtk_container_remove(GTK_CONTAINER(unit_pixmap_table),
+                         more_arrow_pixmap);
+    gtk_widget_unref(more_arrow_pixmap);
   }
-  gtk_container_remove(GTK_CONTAINER(unit_pixmap_table),
-		       more_arrow_pixmap);
-  gtk_widget_unref(more_arrow_pixmap);
 
   populate_unit_pixmap_table();
 
@@ -878,13 +990,37 @@ void enable_menus(bool enable)
   }
 }
 
+/**************************************************************************
+  Workaround for a crash that occurs when a button release event is
+  emitted for a notebook with no pages. See PR#40743.
+  FIXME: Remove this hack once gtk_notebook_button_release() in
+  gtk/gtknotebook.c checks for NULL notebook->cur_page.
+**************************************************************************/
+static gboolean right_notebook_button_release(GtkWidget *widget,
+                                              GdkEventButton *event)
+{
+  if (event->type != GDK_BUTTON_RELEASE) {
+    return FALSE;
+  }
+
+  if (!GTK_IS_NOTEBOOK(widget)
+      || -1 == gtk_notebook_get_current_page(GTK_NOTEBOOK(widget))) {
+    /* Make sure the default gtk handler
+     * does NOT get called in this case. */
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 /**************************************************************************
  do the heavy lifting for the widget setup.
 **************************************************************************/
 static void setup_widgets(void)
 {
   GtkWidget *box, *ebox, *hbox, *sbox, *align, *label;
-  GtkWidget *frame, *table, *table2, *paned, *sw, *text;
+  GtkWidget *frame, *table, *table2, *paned, *hpaned, *sw, *text;
+  GtkWidget *button;
   int i;
   char buf[256];
   struct sprite *sprite;
@@ -990,13 +1126,15 @@ static void setup_widgets(void)
 
   ebox = gtk_event_box_new();
   gtk_widget_add_events(ebox, GDK_BUTTON_PRESS_MASK);
-
-  gtk_box_pack_start(GTK_BOX(vbox), ebox, FALSE, FALSE, 0);
-
-  main_label_info = gtk_label_new("\n\n\n\n");
-  gtk_container_add(GTK_CONTAINER(ebox), main_label_info);
   g_signal_connect(ebox, "button_press_event",
                    G_CALLBACK(show_info_popup), NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), ebox, FALSE, FALSE, 0);
+
+  label = gtk_label_new(NULL);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+  gtk_misc_set_padding(GTK_MISC(label), 2, 2);
+  gtk_container_add(GTK_CONTAINER(ebox), label);
+  main_label_info = label;
 
   /* Production status */
 
@@ -1079,7 +1217,6 @@ static void setup_widgets(void)
   timeout_label = gtk_label_new("");
 
   frame = gtk_frame_new(NULL);
-  gtk_widget_set_size_request(frame, tileset_small_sprite_width(tileset), tileset_small_sprite_height(tileset));
   gtk_table_attach_defaults(GTK_TABLE(table), frame, 4, 10, 1, 2);
   gtk_container_add(GTK_CONTAINER(frame), timeout_label);
 
@@ -1100,8 +1237,11 @@ static void setup_widgets(void)
   unit_info_frame = gtk_frame_new("");
   gtk_box_pack_start(GTK_BOX(avbox), unit_info_frame, FALSE, FALSE, 0);
     
-  unit_info_label = gtk_label_new("\n\n\n");
-  gtk_container_add(GTK_CONTAINER(unit_info_frame), unit_info_label);
+  label = gtk_label_new(NULL);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+  gtk_misc_set_padding(GTK_MISC(label), 2, 2);
+  gtk_container_add(GTK_CONTAINER(unit_info_frame), label);
+  unit_info_label = label;
 
   box = gtk_hbox_new(FALSE,0);
   gtk_box_pack_start(GTK_BOX(avbox), box, FALSE, FALSE, 0);
@@ -1113,7 +1253,6 @@ static void setup_widgets(void)
   gtk_table_set_col_spacings(GTK_TABLE(table), 2);
 
   unit_pixmap_table = table;
-  populate_unit_pixmap_table();
 
   top_notebook = gtk_notebook_new();  
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(top_notebook), GTK_POS_BOTTOM);
@@ -1149,7 +1288,6 @@ static void setup_widgets(void)
                                    |GDK_POINTER_MOTION_MASK
                                    |GDK_SCROLL_MASK);
 
-  gtk_widget_set_size_request(map_canvas, 510, 300);
   gtk_container_add(GTK_CONTAINER(frame), map_canvas);
 
   map_horizontal_scrollbar = gtk_hscrollbar_new(NULL);
@@ -1190,10 +1328,24 @@ static void setup_widgets(void)
   gtk_paned_pack2(GTK_PANED(paned), sbox, TRUE, TRUE);
   avbox = detached_widget_fill(sbox);
 
+  hpaned = gtk_hpaned_new();
+  gtk_box_pack_start(GTK_BOX(avbox), hpaned, TRUE, TRUE, 0);
+  bottom_hpaned = hpaned;
+
   bottom_notebook = gtk_notebook_new();
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(bottom_notebook), GTK_POS_TOP);
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(bottom_notebook), TRUE);
-  gtk_box_pack_start(GTK_BOX(avbox), bottom_notebook, TRUE, TRUE, 0);
+  gtk_paned_pack1(GTK_PANED(hpaned), bottom_notebook, TRUE, TRUE);
+
+  right_notebook = gtk_notebook_new();
+  g_object_ref(right_notebook);
+  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(right_notebook), GTK_POS_TOP);
+  gtk_notebook_set_scrollable(GTK_NOTEBOOK(right_notebook), TRUE);
+  g_signal_connect(right_notebook, "button-release-event",
+                   G_CALLBACK(right_notebook_button_release), NULL);
+  if (split_bottom_notebook) {
+    gtk_paned_pack2(GTK_PANED(hpaned), right_notebook, TRUE, TRUE);
+  }
 
   vbox = gtk_vbox_new(FALSE, 0);
 
@@ -1202,7 +1354,6 @@ static void setup_widgets(void)
 				      GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,
   				 GTK_POLICY_ALWAYS);
-  gtk_widget_set_size_request(sw, 600, 100);
   gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
 
   label = gtk_label_new(_("Chat"));
@@ -1211,6 +1362,8 @@ static void setup_widgets(void)
   text = gtk_text_view_new_with_buffer(message_buffer);
   gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
   gtk_container_add(GTK_CONTAINER(sw), text);
+  g_signal_connect(text, "size-allocate",
+                   G_CALLBACK(main_message_area_size_allocate), NULL);
 
   gtk_widget_set_name(text, "chatline");
 
@@ -1226,18 +1379,28 @@ static void setup_widgets(void)
       " menu.\nNow.. Go give'em hell!") );
 
   /* the chat line */
-  inputline = gtk_entry_new();
-  gtk_box_pack_start(GTK_BOX(vbox), inputline, FALSE, FALSE, 3);
+  hbox = gtk_hbox_new(FALSE, 4);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);
 
+  inputline = gtk_entry_new();
   g_signal_connect(inputline, "activate",
 		   G_CALLBACK(inputline_return), NULL);
   g_signal_connect(inputline, "key_press_event",
                    G_CALLBACK(inputline_handler), NULL);
+  gtk_box_pack_start(GTK_BOX(hbox), inputline, TRUE, TRUE, 0);
+
+  button = gtk_toggle_button_new_with_label(_("Allies Only"));
+  gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+                               allied_chat_only);
+  g_signal_connect(button, "toggled",
+                   G_CALLBACK(allied_chat_button_toggled), NULL);
+  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 2);
+  allied_chat_toggle_button = button;
 
   /* Other things to take care of */
 
   gtk_widget_show_all(gtk_bin_get_child(GTK_BIN(toplevel)));
-  gtk_widget_hide(more_arrow_pixmap);
 
   if (enable_tabs) {
     popup_meswin_dialog(FALSE);
@@ -1426,6 +1589,9 @@ void ui_main(int argc, char **argv)
     gdk_gc_set_foreground(mask_bg_gc, &pixel);
   }
 
+  selection_gc = gdk_gc_new(root_window);
+  gdk_gc_set_function(selection_gc, GDK_XOR);
+
   tileset_init(tileset);
   tileset_load_tiles(tileset);
 
@@ -1684,6 +1850,10 @@ void set_unit_icon(int idx, struct unit *punit)
     unit_ids[idx] = punit ? punit->id : 0;
   }
 
+  if (!w) {
+    return;
+  }
+
   gtk_pixcomm_freeze(GTK_PIXCOMM(w));
 
   if (punit) {
@@ -1704,6 +1874,10 @@ void set_unit_icons_more_arrow(bool onoff)
 {
   static bool showing = FALSE;
 
+  if (!more_arrow_pixmap) {
+    return;
+  }
+
   if (onoff && !showing) {
     gtk_widget_show(more_arrow_pixmap);
     showing = TRUE;
@@ -1950,3 +2124,65 @@ void add_idle_callback(void (callback)(void *), void *data)
   cb->data = data;
   gtk_idle_add(idle_callback_wrapper, cb);
 }
+
+/****************************************************************************
+  Option callback for the 'split_bottom_notebook' option.
+****************************************************************************/
+static void split_bottom_notebook_callback(struct client_option *op)
+{
+  popdown_meswin_dialog();
+  if (*op->p_bool_value) {
+    gtk_paned_pack2(GTK_PANED(bottom_hpaned), right_notebook, TRUE, TRUE);
+    gtk_widget_show_all(right_notebook);
+  } else {
+    gtk_container_remove(GTK_CONTAINER(bottom_hpaned), right_notebook);
+  }
+  popup_meswin_dialog(FALSE);
+}
+
+/****************************************************************************
+  Option callback for the 'allied_chat_only' option. This updates the state
+  of the associated toggle button.
+****************************************************************************/
+static void allied_chat_only_callback(struct client_option *op)
+{
+  GtkWidget *button;
+
+  button = allied_chat_toggle_button;
+  g_return_if_fail(button != NULL);
+  g_return_if_fail(GTK_IS_TOGGLE_BUTTON(button));
+
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+                               *op->p_bool_value);
+}
+
+/**************************************************************************
+  Set the chatline buttons to reflect the state of the game and current
+  client options. This function should be called on game start.
+**************************************************************************/
+void refresh_chat_buttons(void)
+{
+  GtkWidget *button;
+
+  button = allied_chat_toggle_button;
+  g_return_if_fail(button != NULL);
+  g_return_if_fail(GTK_IS_TOGGLE_BUTTON(button));
+
+  /* Hide the "Allies Only" button for local games. */
+  if (is_server_running()) {
+    gtk_widget_hide(button);
+  } else {
+    gtk_widget_show(button);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
+                                 allied_chat_only);
+  }
+}
+
+/**************************************************************************
+  Handle a toggle of the "Allies Only" chat button.
+**************************************************************************/
+static void allied_chat_button_toggled(GtkToggleButton *button,
+                                       gpointer user_data)
+{
+  allied_chat_only = gtk_toggle_button_get_active(button);
+}
