@@ -64,6 +64,9 @@ static GtkWidget *scenario_description;
 static GtkListStore *load_store, *scenario_store,
   *nation_store, *meta_store, *lan_store; 
 
+static GtkListStore *server_playerlist_store;
+static GtkWidget *server_playerlist_view;
+
 static GtkTreeSelection *load_selection, *scenario_selection,
   *nation_selection, *meta_selection, *lan_selection;
 
@@ -72,14 +75,15 @@ static enum client_pages old_page = -1;
 static void set_page_callback(GtkWidget *w, gpointer data);
 static void update_nation_page(struct packet_game_load *packet);
 
-static guint scan_timer = 0;
-struct server_scan *lan, *meta;
+static guint meta_scan_timer, lan_scan_timer;
+static struct server_scan *meta_scan, *lan_scan;
 
 static GtkWidget *statusbar, *statusbar_frame;
 static GQueue *statusbar_queue;
 static guint statusbar_timer = 0;
 
 static GtkWidget *ruleset_combo;
+static GtkWidget *start_page_entry;
 
 /**************************************************************************
   spawn a server, if there isn't one, using the default settings.
@@ -173,19 +177,17 @@ static void ggz_login(void)
 **************************************************************************/
 GtkWidget *create_main_page(void)
 {
-  GtkWidget *align, *box, *sbox, *bbox, *frame, *image;
-
-  GtkWidget *button;
+  GtkWidget *widget, *align, *vbox, *frame, *image, *button, *table;
   GtkSizeGroup *size;
 
   size = gtk_size_group_new(GTK_SIZE_GROUP_BOTH);
 
-  box = gtk_vbox_new(FALSE, 6);
-  gtk_container_set_border_width(GTK_CONTAINER(box), 4);
+  vbox = gtk_vbox_new(FALSE, 0);
+  widget = vbox;
 
   align = gtk_alignment_new(0.5, 0.0, 0.0, 0.0);
   gtk_container_set_border_width(GTK_CONTAINER(align), 18);
-  gtk_box_pack_start(GTK_BOX(box), align, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
 
   frame = gtk_frame_new(NULL);
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
@@ -193,61 +195,9 @@ GtkWidget *create_main_page(void)
 
   image = gtk_image_new_from_file(tileset_main_intro_filename(tileset));
   g_signal_connect_after(image, "expose_event",
-      G_CALLBACK(intro_expose), NULL);
+                         G_CALLBACK(intro_expose), NULL);
   gtk_container_add(GTK_CONTAINER(frame), image);
 
-  align = gtk_alignment_new(0.5, 0.0, 0.0, 0.0);
-  gtk_box_pack_start(GTK_BOX(box), align, FALSE, FALSE, 0);
-
-  sbox = gtk_vbox_new(FALSE, 18);
-  gtk_container_add(GTK_CONTAINER(align), sbox);
-
-  bbox = gtk_vbox_new(FALSE, 6);
-  gtk_container_add(GTK_CONTAINER(sbox), bbox);
-
-  button = gtk_button_new_with_mnemonic(_("Start _New Game"));
-  gtk_size_group_add_widget(size, button);
-  gtk_container_add(GTK_CONTAINER(bbox), button);
-  g_signal_connect(button, "clicked",
-      G_CALLBACK(start_new_game_callback), NULL);
-
-  button = gtk_button_new_with_mnemonic(_("Start _Scenario Game"));
-  gtk_size_group_add_widget(size, button);
-  gtk_container_add(GTK_CONTAINER(bbox), button);
-  g_signal_connect(button, "clicked",
-      G_CALLBACK(start_scenario_callback), NULL);
-
-  button = gtk_button_new_with_mnemonic(_("_Load Saved Game"));
-  gtk_size_group_add_widget(size, button);
-  gtk_container_add(GTK_CONTAINER(bbox), button);
-  g_signal_connect(button, "clicked",
-      G_CALLBACK(load_saved_game_callback), NULL);
-
-  bbox = gtk_vbox_new(FALSE, 6);
-  gtk_container_add(GTK_CONTAINER(sbox), bbox);
-
-  button = gtk_button_new_with_mnemonic(_("C_onnect to Network Game"));
-  gtk_size_group_add_widget(size, button);
-  gtk_container_add(GTK_CONTAINER(bbox), button);
-  g_signal_connect(button, "clicked",
-      G_CALLBACK(set_page_callback), GUINT_TO_POINTER(PAGE_NETWORK));
-
-#ifdef GGZ_GTK
-  button = gtk_button_new_with_mnemonic(_("Connect to Gaming _Zone"));
-  gtk_size_group_add_widget(size, button);
-  gtk_container_add(GTK_CONTAINER(bbox), button);
-  g_signal_connect(button, "clicked", ggz_login, NULL);
-#endif
-
-  bbox = gtk_vbox_new(FALSE, 6);
-  gtk_container_add(GTK_CONTAINER(sbox), bbox);
-
-  button = gtk_button_new_from_stock(GTK_STOCK_QUIT);
-  gtk_size_group_add_widget(size, button);
-  gtk_container_add(GTK_CONTAINER(bbox), button);
-  g_signal_connect(button, "clicked",
-      G_CALLBACK(gtk_main_quit), NULL);
-
 #if IS_BETA_VERSION
   {
     GtkWidget *label;
@@ -256,11 +206,57 @@ GtkWidget *create_main_page(void)
     gtk_widget_set_name(label, "beta label");
     gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
     gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
-    gtk_container_add(GTK_CONTAINER(sbox), label);
+    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   }
 #endif
 
-  return box;
+  table = gtk_table_new(3, 2, TRUE);
+  align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
+  gtk_box_pack_start(GTK_BOX(vbox), align, TRUE, TRUE, 12);
+
+  gtk_table_set_row_spacings(GTK_TABLE(table), 8);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 18);
+  gtk_container_add(GTK_CONTAINER(align), table);
+
+  button = gtk_button_new_with_mnemonic(_("Start _New Game"));
+  gtk_size_group_add_widget(size, button);
+  gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 0, 1);
+  g_signal_connect(button, "clicked",
+                   G_CALLBACK(start_new_game_callback), NULL);
+
+  button = gtk_button_new_with_mnemonic(_("Start _Scenario Game"));
+  gtk_size_group_add_widget(size, button);
+  gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 1, 2);
+  g_signal_connect(button, "clicked",
+                   G_CALLBACK(start_scenario_callback), NULL);
+
+  button = gtk_button_new_with_mnemonic(_("_Load Saved Game"));
+  gtk_size_group_add_widget(size, button);
+  gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 2, 3);
+  g_signal_connect(button, "clicked",
+                   G_CALLBACK(load_saved_game_callback), NULL);
+
+  button = gtk_button_new_with_mnemonic(_("C_onnect to Network Game"));
+  gtk_size_group_add_widget(size, button);
+  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 0, 1);
+  g_signal_connect(button, "clicked",
+                   G_CALLBACK(set_page_callback),
+                   GUINT_TO_POINTER(PAGE_NETWORK));
+
+#ifdef GGZ_GTK
+  button = gtk_button_new_with_mnemonic(_("Connect to Gaming _Zone"));
+  gtk_size_group_add_widget(size, button);
+  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1, 2);
+  g_signal_connect(button, "clicked", ggz_login, NULL);
+#endif
+
+  button = gtk_button_new_from_stock(GTK_STOCK_QUIT);
+  gtk_size_group_add_widget(size, button);
+  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2, 3);
+  g_signal_connect(button, "clicked",
+                   G_CALLBACK(gtk_main_quit), NULL);
+
+  return widget;
 }
 
 
@@ -276,84 +272,106 @@ static GtkWidget *network_confirm_password_label, *network_confirm_password;
 /**************************************************************************
   update a server list.
 **************************************************************************/
-static void update_server_list(GtkTreeSelection *selection,
-			       GtkListStore *store, struct server_list *list)
+static void update_server_list(enum server_scan_type sstype,
+                               const struct server_list *list)
 {
-  const gchar *host, *port;
+  GtkTreeSelection *sel = NULL;
+  GtkTreeView *view;
+  GtkTreeIter it;
+  GtkListStore *store;
+  const gchar *host, *portstr;
+  int port;
 
-  host = gtk_entry_get_text(GTK_ENTRY(network_host));
-  port = gtk_entry_get_text(GTK_ENTRY(network_port));
+  switch (sstype) {
+  case SERVER_SCAN_LOCAL:
+    sel = lan_selection;
+    break;
+  case SERVER_SCAN_GLOBAL:
+    sel = meta_selection;
+    break;
+  default:
+    break;
+  }
 
+  if (!sel) {
+    return;
+  }
+
+  view = gtk_tree_selection_get_tree_view(sel);
+  store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
   gtk_list_store_clear(store);
 
   if (!list) {
     return;
   }
 
+  host = gtk_entry_get_text(GTK_ENTRY(network_host));
+  portstr = gtk_entry_get_text(GTK_ENTRY(network_port));
+  port = atoi(portstr);
+
   server_list_iterate(list, pserver) {
-    GtkTreeIter it;
-    gchar *row[6];
-
     gtk_list_store_append(store, &it);
-
-    row[0] = pserver->host;
-    row[1] = pserver->port;
-    row[2] = pserver->version;
-    row[3] = _(pserver->state);
-    row[4] = pserver->nplayers;
-    row[5] = pserver->message;
-
     gtk_list_store_set(store, &it,
-	0, row[0], 1, row[1], 2, row[2],
-	3, row[3], 4, row[4], 5, row[5], -1);
-
-    if (strcmp(host, pserver->host) == 0 && strcmp(port, pserver->port) == 0) {
-      gtk_tree_selection_select_iter(selection, &it);
+                       0, pserver->host,
+                       1, pserver->port,
+                       2, pserver->version,
+                       3, _(pserver->state),
+                       4, pserver->nplayers,
+                       5, pserver->message,
+                       -1);
+    if (strcmp(host, pserver->host) == 0 && port == pserver->port) {
+      gtk_tree_selection_select_iter(sel, &it);
     }
   } server_list_iterate_end;
 }
 
 /**************************************************************************
-  this function frees the list of LAN servers on timeout destruction. 
+  Free the server scans.
 **************************************************************************/
-static void get_server_scan_destroy(gpointer data)
+static void destroy_server_scans(void)
 {
-  if (meta) {
-    server_scan_finish(meta);
-    meta = NULL;
+  if (meta_scan) {
+    server_scan_finish(meta_scan);
+    meta_scan = NULL;
   }
-  if (lan) {
-    server_scan_finish(lan);
-    lan = NULL;
+  if (meta_scan_timer != 0) {
+    g_source_remove(meta_scan_timer);
+    meta_scan_timer = 0;
+  }
+  if (lan_scan) {
+    server_scan_finish(lan_scan);
+    lan_scan = NULL;
+  }
+  if (lan_scan_timer != 0) {
+    g_source_remove(lan_scan_timer);
+    lan_scan_timer = 0;
   }
-  scan_timer = 0;
 }
 
 /**************************************************************************
-  this function updates the list of servers every so often.
+  This function updates the list of servers every so often.
 **************************************************************************/
-static gboolean get_server_scan_list(gpointer data)
+static gboolean check_server_scan(gpointer data)
 {
-  struct server_list *server_list;
+  struct server_scan *scan = data;
+  const struct server_list *servers;
+  enum server_scan_status stat;
 
-  if (!meta && !lan) {
+  if (!scan) {
     return FALSE;
   }
 
-  if (lan) {
-    server_list = server_scan_get_servers(lan);
-    if (server_list) {
-      update_server_list(lan_selection, lan_store, server_list);
-    }
+  stat = server_scan_poll(scan);
+  if (stat >= SCAN_STATUS_PARTIAL) {
+    enum server_scan_type type;
+    type = server_scan_get_type(scan);
+    servers = server_scan_get_list(scan);
+    update_server_list(type, servers);
   }
 
-  if (meta) {
-    server_list = server_scan_get_servers(meta);
-    if (server_list) {
-      update_server_list(meta_selection, meta_store, server_list);
-    }
+  if (stat == SCAN_STATUS_ERROR || stat == SCAN_STATUS_DONE) {
+    return FALSE;
   }
-
   return TRUE;
 }
 
@@ -364,16 +382,16 @@ static void server_scan_error(struct server_scan *scan,
 			      const char *message)
 {
   append_output_window(message);
-  freelog(LOG_NORMAL, "%s", message);
+  freelog(LOG_ERROR, "%s", message);
 
   switch (server_scan_get_type(scan)) {
   case SERVER_SCAN_LOCAL:
-    server_scan_finish(lan);
-    lan = NULL;
+    server_scan_finish(lan_scan);
+    lan_scan = NULL;
     break;
   case SERVER_SCAN_GLOBAL:
-    server_scan_finish(meta);
-    meta = NULL;
+    server_scan_finish(meta_scan);
+    meta_scan = NULL;
     break;
   case SERVER_SCAN_LAST:
     break;
@@ -381,25 +399,17 @@ static void server_scan_error(struct server_scan *scan,
 }
 
 /**************************************************************************
-  update the metaserver and lan server lists.
+  Stop and restart the metaserver and lan server scans.
 **************************************************************************/
 static void update_network_lists(void)
 {
-  if (scan_timer != 0) {
-    g_source_remove(scan_timer);
-  }
+  destroy_server_scans();
 
-  if (!meta) {
-    meta = server_scan_begin(SERVER_SCAN_GLOBAL, server_scan_error);
-  }
+  meta_scan = server_scan_begin(SERVER_SCAN_GLOBAL, server_scan_error);
+  meta_scan_timer = g_timeout_add(200, check_server_scan, meta_scan);
 
-  if (!lan) {
-    lan = server_scan_begin(SERVER_SCAN_LOCAL, server_scan_error);
-  }
-
-  scan_timer = g_timeout_add_full(G_PRIORITY_DEFAULT, 100,
-                                  get_server_scan_list,
-                                  NULL, get_server_scan_destroy);
+  lan_scan = server_scan_begin(SERVER_SCAN_LOCAL, server_scan_error);
+  lan_scan_timer = g_timeout_add(500, check_server_scan, lan_scan);
 }
 
 /**************************************************************************
@@ -644,29 +654,70 @@ static void network_activate_callback(GtkTreeView *view,
 }
 
 /**************************************************************************
-  sets the host, port of the selected server.
+  Fills the server player list with the players in the given server, or
+  clears it if there is no player data.
+**************************************************************************/
+static void update_server_playerlist(const struct server *pserver)
+{
+  GtkListStore *store;
+  GtkTreeIter iter;
+  int n, i;
+
+  store = server_playerlist_store;
+  g_return_if_fail(store != NULL);
+
+  gtk_list_store_clear(store);
+  if (!pserver || !pserver->players) {
+    return;
+  }
+
+  n = pserver->nplayers;
+  for (i = 0; i < n; i++) {
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter,
+                       0, pserver->players[i].name,
+                       1, pserver->players[i].type,
+                       2, pserver->players[i].host,
+                       3, pserver->players[i].nation,
+                       -1);
+  }
+}
+
+/**************************************************************************
+  Sets the host, port and player list of the selected server.
 **************************************************************************/
 static void network_list_callback(GtkTreeSelection *select, gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
-  char *host, *port;
+  const char *host;
+  int port;
+  char portstr[32];
+  const struct server *pserver = NULL;
 
   if (!gtk_tree_selection_get_selected(select, &model, &it)) {
     return;
   }
 
-  /* only one server can be selected in either list. */
   if (select == meta_selection) {
-    gtk_tree_selection_unselect_all(lan_selection);
-  } else {
-    gtk_tree_selection_unselect_all(meta_selection);
+    GtkTreePath *path;
+    const struct server_list *servers;
+
+    servers = server_scan_get_list(meta_scan);
+    path = gtk_tree_model_get_path(model, &it);
+    if (servers && path) {
+      gint pos = gtk_tree_path_get_indices(path)[0];
+      pserver = server_list_get(servers, pos);
+    }
+    gtk_tree_path_free(path);
   }
+  update_server_playerlist(pserver);
 
   gtk_tree_model_get(model, &it, 0, &host, 1, &port, -1);
 
   gtk_entry_set_text(GTK_ENTRY(network_host), host);
-  gtk_entry_set_text(GTK_ENTRY(network_port), port);
+  my_snprintf(portstr, sizeof(portstr), "%d", port);
+  gtk_entry_set_text(GTK_ENTRY(network_port), portstr);
 }
 
 /**************************************************************************
@@ -692,27 +743,10 @@ static void update_network_page(void)
 **************************************************************************/
 GtkWidget *create_network_page(void)
 {
-  GtkWidget *salign, *box, *sbox, *bbox, *notebook;
-
-  GtkWidget *button, *label, *view, *sw;
-  GtkCellRenderer *rend;
+  GtkWidget *box, *sbox, *bbox, *hbox, *notebook;
+  GtkWidget *button, *label, *view, *sw, *table;
   GtkTreeSelection *selection;
-
-  GtkWidget *table;
-
-  static const char *titles[] = {
-    N_("Server Name"),
-    N_("Port"),
-    N_("Version"),
-    N_("Status"),
-    N_("Players"),
-    N_("Comment")
-  };
-  static bool titles_done;
-
-  int i;
-
-  intl_slist(ARRAY_SIZE(titles), titles, &titles_done);
+  GtkListStore *store;
 
   box = gtk_vbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(box), 4);
@@ -721,8 +755,12 @@ GtkWidget *create_network_page(void)
   gtk_container_add(GTK_CONTAINER(box), notebook);
 
   /* LAN pane. */
-  lan_store = gtk_list_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
-      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+  lan_store = gtk_list_store_new(6, G_TYPE_STRING, /* host */
+                                 G_TYPE_INT,       /* port */
+                                 G_TYPE_STRING,    /* version */
+                                 G_TYPE_STRING,    /* state */
+                                 G_TYPE_INT,       /* nplayers */
+                                 G_TYPE_STRING);   /* message */
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(lan_store));
   g_object_unref(lan_store);
@@ -738,11 +776,12 @@ GtkWidget *create_network_page(void)
   g_signal_connect(selection, "changed",
                    G_CALLBACK(network_list_callback), NULL);
 
-  rend = gtk_cell_renderer_text_new();
-  for (i = 0; i < ARRAY_SIZE(titles); i++) {
-    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
-	-1, titles[i], rend, "text", i, NULL);
-  }
+  add_treeview_column(view, _("Server Name"), G_TYPE_STRING, 0);
+  add_treeview_column(view, _("Port"), G_TYPE_INT, 1);
+  add_treeview_column(view, _("Version"), G_TYPE_STRING, 2);
+  add_treeview_column(view, _("Status"), G_TYPE_STRING, 3);
+  add_treeview_column(view, _("Players"), G_TYPE_INT, 4);
+  add_treeview_column(view, _("Comment"), G_TYPE_STRING, 5);
 
   label = gtk_label_new_with_mnemonic(_("Local _Area Network"));
 
@@ -757,8 +796,12 @@ GtkWidget *create_network_page(void)
 
 
   /* Metaserver pane. */
-  meta_store = gtk_list_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
-      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+  meta_store = gtk_list_store_new(6, G_TYPE_STRING, /* host */
+                                  G_TYPE_INT,       /* port */
+                                  G_TYPE_STRING,    /* version */
+                                  G_TYPE_STRING,    /* state */
+                                  G_TYPE_INT,       /* nplayers */
+                                  G_TYPE_STRING);   /* message */
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(meta_store));
   g_object_unref(meta_store);
@@ -774,11 +817,12 @@ GtkWidget *create_network_page(void)
   g_signal_connect(selection, "changed",
                    G_CALLBACK(network_list_callback), NULL);
 
-  rend = gtk_cell_renderer_text_new();
-  for (i = 0; i < ARRAY_SIZE(titles); i++) {
-    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
-	-1, titles[i], rend, "text", i, NULL);
-  }
+  add_treeview_column(view, _("Server Name"), G_TYPE_STRING, 0);
+  add_treeview_column(view, _("Port"), G_TYPE_INT, 1);
+  add_treeview_column(view, _("Version"), G_TYPE_STRING, 2);
+  add_treeview_column(view, _("Status"), G_TYPE_STRING, 3);
+  add_treeview_column(view, _("Players"), G_TYPE_INT, 4);
+  add_treeview_column(view, _("Comment"), G_TYPE_STRING, 5);
 
   label = gtk_label_new_with_mnemonic(_("Internet _Metaserver"));
 
@@ -789,21 +833,24 @@ GtkWidget *create_network_page(void)
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(sw), view);
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw, label);
+  if (metaserver_tab_first) {
+    gtk_notebook_prepend_page(GTK_NOTEBOOK(notebook), sw, label);
+  } else {
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw, label);
+  }
 
   /* Bottom part of the page, outside the inner notebook. */
   sbox = gtk_vbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(box), sbox, FALSE, FALSE, 0);
 
-  salign = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-  gtk_box_pack_start(GTK_BOX(sbox), salign, FALSE, FALSE, 18);
+  hbox = gtk_hbox_new(FALSE, 12);
+  gtk_box_pack_start(GTK_BOX(sbox), hbox, FALSE, FALSE, 8);
 
   table = gtk_table_new(6, 2, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE(table), 2);
   gtk_table_set_col_spacings(GTK_TABLE(table), 12);
   gtk_table_set_row_spacing(GTK_TABLE(table), 2, 12);
-  gtk_widget_set_size_request(table, 400, -1);
-  gtk_container_add(GTK_CONTAINER(salign), table);
+  gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
 
   network_host = gtk_entry_new();
   g_signal_connect(network_host, "activate",
@@ -892,11 +939,33 @@ GtkWidget *create_network_page(void)
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
 		   GTK_FILL, GTK_FILL, 0, 0);
 
+  /* Server player list. */
+  store = gtk_list_store_new(4, G_TYPE_STRING,
+                             G_TYPE_STRING,
+                             G_TYPE_STRING,
+                             G_TYPE_STRING);
+  server_playerlist_store = store;
+
+  view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+  add_treeview_column(view, _("Name"), G_TYPE_STRING, 0);
+  add_treeview_column(view, _("Type"), G_TYPE_STRING, 1);
+  add_treeview_column(view, _("Host"), G_TYPE_STRING, 2);
+  add_treeview_column(view, _("Nation"), G_TYPE_STRING, 3);
+  server_playerlist_view = view;
+
+  sw = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
+				      GTK_SHADOW_ETCHED_IN);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_container_add(GTK_CONTAINER(sw), view);
+  gtk_box_pack_start(GTK_BOX(hbox), sw, TRUE, TRUE, 0);
+
 
   bbox = gtk_hbutton_box_new();
   gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
   gtk_box_set_spacing(GTK_BOX(bbox), 12);
-  gtk_box_pack_start(GTK_BOX(sbox), bbox, FALSE, FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(sbox), bbox, FALSE, FALSE, 0);
 
   button = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
   gtk_container_add(GTK_CONTAINER(bbox), button);
@@ -1529,8 +1598,8 @@ GtkWidget *create_start_page(void)
 
   rend = gtk_cell_renderer_text_new();
   gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
-					      -1, _("Leader"), rend,
-					      "text", 3, NULL);
+                                              -1, Q_("?player:Leader"),
+                                              rend, "text", 3, NULL);
 
   rend = gtk_cell_renderer_text_new();
   col = gtk_tree_view_column_new_with_attributes(_("Nation"),
@@ -1551,7 +1620,8 @@ GtkWidget *create_start_page(void)
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
 				      GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-				 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+                                 GTK_POLICY_AUTOMATIC,
+                                 GTK_POLICY_ALWAYS);
   gtk_widget_set_size_request(sw, -1, 200);
   gtk_container_add(GTK_CONTAINER(sw), view);
   gtk_box_pack_start(GTK_BOX(sbox), sw, TRUE, TRUE, 0);
@@ -1561,7 +1631,8 @@ GtkWidget *create_start_page(void)
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
 				      GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-				 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+                                 GTK_POLICY_AUTOMATIC,
+                                 GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(box), sw, TRUE, TRUE, 0);
 
   text = gtk_text_view_new_with_buffer(message_buffer);
@@ -1582,6 +1653,7 @@ GtkWidget *create_start_page(void)
       G_CALLBACK(inputline_handler), NULL);
   g_signal_connect(entry, "activate",
       G_CALLBACK(inputline_return), NULL);
+  start_page_entry = entry;
 
   bbox = gtk_hbutton_box_new();
   gtk_box_set_spacing(GTK_BOX(bbox), 12);
@@ -2166,10 +2238,7 @@ void set_client_page(enum client_pages page)
   case PAGE_LOAD:
     break;
   case PAGE_NETWORK:
-    if (scan_timer != 0) {
-      g_source_remove(scan_timer);
-      scan_timer = 0;
-    }
+    destroy_server_scans();
     break;
   case PAGE_GAME:
     enable_menus(FALSE);
@@ -2193,6 +2262,7 @@ void set_client_page(enum client_pages page)
   case PAGE_NATION:
     break;
   case PAGE_GAME:
+    reset_unit_table();
     enable_menus(TRUE);
     break;
   case PAGE_LOAD:
@@ -2223,7 +2293,11 @@ void set_client_page(enum client_pages page)
 
   switch (new_page) {
   case PAGE_MAIN:
+    break;
   case PAGE_START:
+    gtk_widget_grab_focus(start_page_entry);
+    chatline_scroll_to_bottom();
+    break;
   case PAGE_GGZ:
     break;
   case PAGE_NATION:
@@ -2236,6 +2310,8 @@ void set_client_page(enum client_pages page)
     gtk_tree_view_focus(gtk_tree_selection_get_tree_view(scenario_selection));
     break;
   case PAGE_GAME:
+    chatline_scroll_to_bottom();
+    refresh_chat_buttons();
     break;
   case PAGE_NETWORK:
     update_network_lists();
