@@ -819,7 +819,7 @@ static void economy_command_callback(struct gui_dialog *dlg, int response,
       NULL,
       GTK_DIALOG_DESTROY_WITH_PARENT,
       GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
-      buf);
+      "%s", buf);
   setup_dialog(shell, gui_dialog_get_toplevel(dlg));
 
   g_signal_connect(shell, "response", G_CALLBACK(gtk_widget_destroy), NULL);
@@ -1178,7 +1178,7 @@ void activeunits_report_dialog_update(void)
 {
   struct repoinfo {
     int active_count;
-    int upkeep[O_COUNT];
+    int upkeep[O_MAX];
     int building_count;
   };
 
