@@ -215,7 +215,7 @@ void handle_server_join_reply(bool you_can_join, char *message,
     aconnection.id = 0;
     if (auto_connect) {
       freelog(LOG_NORMAL, _("You were rejected from the game...."));
-      freelog(LOG_NORMAL, msg);
+      freelog(LOG_NORMAL, "%s", msg);
     }
     gui_server_connect();
     if (!with_ggz) {
@@ -889,10 +889,6 @@ void handle_start_phase(int phase)
   game.info.phase = phase;
 
   if (game.player_ptr && is_player_phase(game.player_ptr, phase)) {
-    /* HACK: this is updated by the player packet too; we update it here
-     * so the turn done button state will be set properly. */
-    game.player_ptr->phase_done = FALSE;
-
     agents_start_turn();
     non_ai_unit_focus = FALSE;
 
@@ -1233,6 +1229,7 @@ static bool handle_unit_packet_common(struct unit *packet_unit)
 	   && unit_owner(punit) == game.player_ptr
 	   && !unit_has_orders(punit)
 	   && can_client_issue_orders()
+	   && popup_caravan_arrival
 	   && (unit_can_help_build_wonder_here(punit)
 	       || unit_can_est_traderoute_here(punit))) {
 	  process_caravan_arrival(punit);
@@ -1417,7 +1414,7 @@ void handle_map_info(int xsize, int ysize, int topology_id)
 
   map_allocate();
   init_client_goto();
-  init_mapview_decorations();
+  mapdeco_init();
 
   generate_citydlg_dimensions();
 
@@ -1443,10 +1440,6 @@ void handle_game_info(struct packet_game_info *pinfo)
     = government_by_number(game.info.government_when_anarchy_id);
   game.player_ptr = player_by_number(game.info.player_idx);
 
-  if (C_S_PREPARING == client_state()) {
-    /* FIXME: only for change in nations */
-    popdown_races_dialog();
-  }
   boot_help = (can_client_change_view()
 	       && game.info.spacerace != pinfo->spacerace);
   if (game.info.timeout != 0 && pinfo->seconds_to_phasedone >= 0) {
@@ -1518,7 +1511,7 @@ void start_revolution(void)
 void handle_player_info(struct packet_player_info *pinfo)
 {
   int i;
-  bool poptechup, new_tech = FALSE;
+  bool poptechup, new_tech = FALSE, phase_done_changed;
   char msg[MAX_LEN_MSG];
   struct player *pplayer = &game.players[pinfo->playerno];
   struct player_research* research;
@@ -1656,10 +1649,7 @@ void handle_player_info(struct packet_player_info *pinfo)
 
   pplayer->is_ready = pinfo->is_ready;
 
-  if (pplayer == game.player_ptr
-      && pplayer->phase_done != pinfo->phase_done) {
-    update_turn_done_button_state();
-  }
+  phase_done_changed = (pplayer->phase_done != pinfo->phase_done);
   pplayer->phase_done = pinfo->phase_done;
 
   pplayer->nturns_idle=pinfo->nturns_idle;
@@ -1669,6 +1659,9 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->revolution_finishes = pinfo->revolution_finishes;
   pplayer->ai.skill_level = pinfo->ai_skill_level;
 
+  if (pplayer == game.player_ptr && phase_done_changed) {
+    update_turn_done_button_state();
+  }
   update_players_dialog();
   update_worklist_report_dialog();
   upgrade_canvas_clipboard();
@@ -2128,20 +2121,8 @@ void handle_tile_info(struct packet_tile_info *packet)
     unit_list_unlink_all(ptile->units);
   }
 
-  /* update continents */
-  if (ptile->continent != packet->continent && ptile->continent != 0
-      && packet->continent > 0) {
-    /* We're renumbering continents, somebody did a transform.
-     * But we don't care about renumbering oceans since 
-     * num_oceans is not kept at the client. */
-    map.num_continents = 0;
-  }
-
   ptile->continent = packet->continent;
-
-  if (ptile->continent > map.num_continents) {
-    map.num_continents = ptile->continent;
-  }
+  map.num_continents = MAX(ptile->continent, map.num_continents);
 
   if (known_changed || tile_changed) {
     /* 
@@ -2178,6 +2159,12 @@ void handle_tile_info(struct packet_tile_info *packet)
 **************************************************************************/
 void handle_player_remove(int player_id)
 {
+  if (player_id < 0 || player_id >= MAX_NUM_PLAYERS) {
+    freelog(LOG_ERROR, "Illegal player_remove packet from server (%d)",
+            player_id);
+    return;
+  }
+
   client_remove_player(player_id);
   update_conn_list_dialog();
 }
@@ -2191,6 +2178,10 @@ void handle_ruleset_control(struct packet_ruleset_control *packet)
 {
   int i;
 
+  /* The ruleset is going to load new nations. So close
+   * the nation selection dialog if it is open. */
+  popdown_races_dialog();
+
   ruleset_data_free();
 
   ruleset_cache_init();
@@ -2364,7 +2355,15 @@ void handle_ruleset_building(struct packet_ruleset_building *p)
     } impr_type_iterate_end;
   }
 #endif
-  
+
+  b->allows_units = FALSE;
+  unit_type_iterate(ut) {
+    if (ut->impr_requirement == p->id) {
+      b->allows_units = TRUE;
+      break;
+    }
+  } unit_type_iterate_end;
+
   tileset_setup_impr_type(tileset, p->id);
 }
 
