@@ -258,7 +258,7 @@ static void finish_revolution(struct player *pplayer)
       = 100 - pplayer->economic.science - pplayer->economic.tax;
   }
 
-  check_player_government_rates(pplayer);
+  check_player_max_rates(pplayer);
   global_city_refresh(pplayer);
   send_player_info(pplayer, pplayer);
 }
@@ -337,7 +337,7 @@ void handle_player_change_government(struct player *pplayer, int government)
 		     _("Revolution: returning to anarchy."));
   }
 
-  check_player_government_rates(pplayer);
+  check_player_max_rates(pplayer);
   global_city_refresh(pplayer);
   send_player_info(pplayer, pplayer);
 
@@ -414,25 +414,22 @@ The following checks that government rates are acceptable for the present
 form of government. Has to be called when switching governments or when
 toggling from AI to human.
 **************************************************************************/
-void check_player_government_rates(struct player *pplayer)
+void check_player_max_rates(struct player *pplayer)
 {
   struct player_economic old_econ = pplayer->economic;
-  bool changed = FALSE;
-  player_limit_to_government_rates(pplayer);
-  if (pplayer->economic.tax != old_econ.tax) {
-    changed = TRUE;
+
+  pplayer->economic = player_limit_to_max_rates(pplayer);
+  if (old_econ.tax > pplayer->economic.tax) {
     notify_player(pplayer, NULL, E_NEW_GOVERNMENT,
 		  _("Tax rate exceeded the max rate for %s; adjusted."), 
 		  government_name_for_player(pplayer));
   }
-  if (pplayer->economic.science != old_econ.science) {
-    changed = TRUE;
+  if (old_econ.science > pplayer->economic.science) {
     notify_player(pplayer, NULL, E_NEW_GOVERNMENT,
 		  _("Science rate exceeded the max rate for %s; adjusted."), 
 		  government_name_for_player(pplayer));
   }
-  if (pplayer->economic.luxury != old_econ.luxury) {
-    changed = TRUE;
+  if (old_econ.luxury > pplayer->economic.luxury) {
     notify_player(pplayer, NULL, E_NEW_GOVERNMENT,
 		  _("Luxury rate exceeded the max rate for %s; adjusted."), 
 		  government_name_for_player(pplayer));
@@ -865,11 +862,6 @@ static void package_player_common(struct player *plr,
     packet->small_wonders[i] = plr->small_wonders[i];
   }
   packet->science_cost = plr->ai.science_cost;
-
-  packet->gold = plr->economic.gold;
-  packet->government = government_of_player(plr)
-                       ? government_of_player(plr)->index
-                       : -1;
 }
 
 /**************************************************************************
@@ -890,6 +882,7 @@ static void package_player_info(struct player *plr,
   enum plr_info_level info_level;
   enum plr_info_level highest_team_level;
   struct player_research* research = get_player_research(plr);
+  struct government *pgov = NULL;
 
   if (receiver) {
     info_level = player_info_level(plr, receiver);
@@ -916,6 +909,15 @@ static void package_player_info(struct player *plr,
     packet->score = 0;
   }
 
+  if (info_level >= INFO_MEETING) {
+    packet->gold = plr->economic.gold;
+    pgov = government_of_player(plr);
+  } else {
+    packet->gold = 0;
+    pgov = game.government_when_anarchy;
+  }
+  packet->government = pgov ? pgov->index : -1;
+   
   /* Send diplomatic status of the player to everyone they are in
    * contact with. */
   if (info_level >= INFO_EMBASSY
@@ -1118,6 +1120,8 @@ void server_remove_player(struct player *pplayer)
   team_remove_player(pplayer);
   game_remove_player(pplayer);
   game_renumber_players(player_number(pplayer));
+
+  aifill(game.info.aifill);
 }
 
 /**************************************************************************
@@ -1548,7 +1552,7 @@ static struct player *split_player(struct player *pplayer)
     } players_iterate_end;
   }
 
-  player_limit_to_government_rates(pplayer);
+  pplayer->economic = player_limit_to_max_rates(pplayer);
 
   /* copy the maps */
 
@@ -1679,9 +1683,9 @@ void civil_war(struct player *pplayer)
 	  nation_rule_name(nation_of_player(pplayer)),
 	  player_name(cplayer));
   notify_player(pplayer, NULL, E_CIVIL_WAR,
-		   _("Your nation is thrust into civil war, "
-		     " %s is declared the leader of the rebel states."),
-		   player_name(cplayer));
+                _("Your nation is thrust into civil war! "
+                  "%s is declared the leader of the rebel states."),
+                player_name(cplayer));
 
   i = city_list_size(pplayer->cities)/2;   /* number to flip */
   j = city_list_size(pplayer->cities);	    /* number left to process */
