@@ -663,7 +663,9 @@ static void dem_line_item(char *outptr, size_t out_size,
   Other settings callback functions are in settings.c, but this one uses
   static values from this file so it's done separately.
 *************************************************************************/
-bool is_valid_demography(const char *demography, const char **error_string)
+bool is_valid_demography(const char *demography,
+                         struct connection *caller,
+                         const char **error_string)
 {
   int len = strlen(demography), i;
 
@@ -743,10 +745,12 @@ void report_demographics(struct connection *pconn)
     return;
   }
 
-  my_snprintf(civbuf, sizeof(civbuf), _("The %s of the %s (%s)"),
-	      government_name_for_player(pplayer),
-	      nation_plural_for_player(pplayer),
-	      textyear(game.info.year));
+  /* TRANS: <nation adjective> <government name> (<year>).
+   * E.g. "Polish Despotism (200 AD)". */
+  my_snprintf(civbuf, sizeof(civbuf), _("%s %s (%s)"),
+              nation_adjective_for_player(pplayer),
+              government_name_for_player(pplayer),
+              textyear(game.info.year));
 
   buffer[0] = '\0';
   for (i = 0; i < ARRAY_SIZE(rowtable); i++) {
