@@ -865,6 +865,40 @@ static void end_turn(void)
   send_year_to_clients(game.info.year);
 }
 
+/**************************************************************************
+  Generate a default save file name and place it in the provided buffer.
+  The name will be of the form "<prefix>-T<turn>-Y<year><suffix><m>" where:
+
+    <prefix> = game.save_name
+    <turn>   = game.info.turn (zero padded to 3 places)
+    <year>   = game.info.year (not padded and no sign)
+    <suffix> = "BC" or "AD" for negative or positive year resp.
+    <m>      = "m" (for "manual save") if 'is_auto_save' is FALSE
+
+  Returns the number of characters written, or the number of characters
+  that would have been written if truncation occurs.
+**************************************************************************/
+static int generate_save_name(char *buf, int buflen, bool is_auto_save)
+{
+  int nb, year;
+  const char *year_suffix;
+
+  if (game.info.year < 0) {
+    year = -game.info.year;
+    year_suffix = "BC";
+  } else {
+    year = game.info.year;
+    year_suffix = "AD";
+  }
+
+  /* NB: If you change the format here, be sure to update the above
+   * function comment and the help text for the 'savename' setting. */
+  nb = my_snprintf(buf, buflen, "%s-T%03d-Y%d%s%s",
+                   game.save_name, game.info.turn, year,
+                   year_suffix, is_auto_save ? "" : "m");
+  return nb;
+}
+
 /**************************************************************************
 Unconditionally save the game, with specified filename.
 Always prints a message: either save ok, or failed.
@@ -890,10 +924,9 @@ void save_game(char *orig_filename, const char *save_reason)
     *dot = '\0';
   }
 
-  /* If orig_filename is NULL or empty, use "civgame.info.year>m". */
+  /* If orig_filename is NULL or empty, use a generated default name. */
   if (filename[0] == '\0'){
-    my_snprintf(filename, sizeof(filename),
-	"%s%+05dm", game.save_name, game.info.year);
+    generate_save_name(filename, sizeof(filename), FALSE);
   }
   
   timer_cpu = new_timer_start(TIMER_CPU, TIMER_ACTIVE);
@@ -934,6 +967,8 @@ void save_game(char *orig_filename, const char *save_reason)
   freelog(LOG_VERBOSE, "Save time: %g seconds (%g apparent)",
 	  read_timer_seconds_free(timer_cpu),
 	  read_timer_seconds_free(timer_user));
+
+  ggz_game_saved(filename);
 }
 
 /**************************************************************************
@@ -945,8 +980,7 @@ void save_game_auto(const char *save_reason)
 
   assert(strlen(game.save_name)<256);
   
-  my_snprintf(filename, sizeof(filename),
-	      "%s%+05d.sav", game.save_name, game.info.year);
+  generate_save_name(filename, sizeof(filename), TRUE);
   save_game(filename, save_reason);
   save_ppm();
 }
@@ -987,12 +1021,17 @@ void server_quit(void)
   set_server_state(S_S_OVER);
   server_game_free();
   diplhand_free();
+
+  /* Free all the warmap arrays */
+  free_mapqueue();
+
 #ifdef HAVE_AUTH
   if (srvarg.auth_enabled) {
     /* If auth has been initialized */
     auth_free();
   }
 #endif /* HAVE_AUTH */
+
   stdinhand_free();
   close_connections_and_socket();
   exit(EXIT_SUCCESS);
@@ -1532,9 +1571,8 @@ void handle_player_ready(struct player *requestor,
 		    "are ready to start."),
 		  num_ready, num_ready + num_unready);
     } else {
-      notify_conn(NULL, NULL, E_SETTING,
-		  _("All players are ready; starting game."));
-      start_game();
+      /* Check minplayers etc. and then start */
+      start_command(NULL, FALSE, TRUE);
     }
   }
 }
@@ -1550,13 +1588,11 @@ void aifill(int amount)
     return;
   }
 
-  if (amount == 0) {
-    /* Special case for value 0: do nothing. */
-    return;
-  }
-
   amount = MIN(amount, game.info.max_players);
 
+  /* Limit to nations provided by ruleset */
+  amount = MIN(amount, game.playable_nations);
+
   while (game.info.nplayers < amount) {
     const int old_nplayers = game.info.nplayers;
     struct player *pplayer = player_by_number(old_nplayers);
@@ -1589,24 +1625,33 @@ void aifill(int amount)
     send_player_info(pplayer, NULL);
   }
 
-  remove = game.info.nplayers - 1;
-  while (game.info.nplayers > amount && remove >= 0) {
-    struct player *pplayer = player_by_number(remove);
-
-    if (!pplayer->is_connected && !pplayer->was_created) {
-      server_remove_player(pplayer);
+  while (game.info.nplayers > amount) {
+    struct player *pplayer_rem = NULL;
+    remove = game.info.nplayers - 1;
+    while (remove >= 0) {
+      struct player *pplayer = player_by_number(remove);
+      if (!pplayer->is_connected && !pplayer->was_created) {
+        pplayer_rem = pplayer;
+        break;
+      }
+      remove--;
+    }
+    if (pplayer_rem) {
+      server_remove_player(pplayer_rem);
+    } else {
+      break;
     }
-    remove--;
   }
 }
 
 /**************************************************************************
-   generate_players() - Selects a nation for players created with
-   server's "create <PlayerName>" command.  If <PlayerName> matches
-   one of the leader names for some nation, we choose that nation.
-   (I.e. if we issue "create Shaka" then we will make that AI player's
-   nation the Zulus if the Zulus have not been chosen by anyone else.
-   If they have, then we pick an available nation at random.)
+   Selects a nation for players created with "create <PlayerName>", or
+   with "set aifill <X>".
+
+   If <PlayerName> matches one of the leader names for some nation,
+   choose that nation.  For example, when the Zulus have not been chosen
+   by anyone else, "create Shaka" will make that AI player's nation the
+   Zulus.  Otherwise, pick an available nation at random.
 
    If the AI player name is one of the leader names for the AI player's
    nation, the player sex is set to the sex for that leader, else it
@@ -1615,8 +1660,6 @@ void aifill(int amount)
 **************************************************************************/
 static void generate_players(void)
 {
-  char leader_name[MAX_LEN_NAME];
-
   /* Select nations for AI players generated with server
    * 'create <name>' command
    */
@@ -1649,11 +1692,17 @@ static void generate_players(void)
 
     pplayer->city_style = city_style_of_nation(nation_of_player(pplayer));
 
-    pick_random_player_name(nation_of_player(pplayer), leader_name);
-    sz_strlcpy(pplayer->name, leader_name);
+    /* don't change the name of a created player */
+    if (!pplayer->was_created) {
+      char leader_name[MAX_LEN_NAME];
 
-    if (check_nation_leader_name(nation_of_player(pplayer), leader_name)) {
-      pplayer->is_male = get_nation_leader_sex(nation_of_player(pplayer), leader_name);
+      pick_random_player_name(nation_of_player(pplayer), leader_name);
+      sz_strlcpy(pplayer->name, leader_name);
+    }
+
+    if (check_nation_leader_name(nation_of_player(pplayer), pplayer->name)) {
+      pplayer->is_male = get_nation_leader_sex(nation_of_player(pplayer),
+                                               pplayer->name);
     } else {
       pplayer->is_male = (myrand(2) == 1);
     }
@@ -1752,7 +1801,7 @@ static void thaw_clients(void)
 }
 
 /**************************************************************************
-  Send PACKET_BEGIN_TURN to all clients capable of handling it.
+  Send PACKET_BEGIN_TURN to all clients.
 **************************************************************************/
 static void send_begin_turn(void)
 {
@@ -1762,7 +1811,7 @@ static void send_begin_turn(void)
 }
 
 /**************************************************************************
-  Send PACKET_END_TURN to all clients capable of handling it.
+  Send PACKET_END_TURN to all clients.
 **************************************************************************/
 static void send_end_turn(void)
 {
@@ -2057,8 +2106,8 @@ static void srv_ready(void)
 
     players_iterate(pplayer) {
       player_map_allocate(pplayer);
-      init_tech(pplayer);
-      player_limit_to_government_rates(pplayer);
+      init_tech(pplayer, TRUE);
+      pplayer->economic = player_limit_to_max_rates(pplayer);
       pplayer->economic.gold = game.info.gold;
     } players_iterate_end;
 
