@@ -784,7 +784,7 @@ static bool unit_bombard(struct unit *punit, struct tile *ptile)
   }
 
   if (maybe_make_veteran(punit)) {
-    notify_unit_experience(punit, FALSE);
+    notify_unit_experience(punit);
   }
 
   send_unit_info(NULL, punit);
@@ -903,7 +903,7 @@ static void handle_unit_attack_request(struct unit *punit, struct unit *pdefende
 		  nation_adjective_for_player(unit_owner(plooser)),
 		  unit_name_translation(plooser));
     if (vet) {
-      notify_unit_experience(pwinner, TRUE);
+      notify_unit_experience(pwinner);
     }
     notify_player(unit_owner(plooser), def_tile, E_UNIT_LOST_ATT,
 		  /* TRANS: "... Cannon ... the Polish Destroyer." */
