@@ -40,6 +40,7 @@
 #ifdef WIN32_NATIVE
 #include <windows.h>
 #include <lmcons.h>	/* UNLEN */
+#include <shlobj.h>
 #endif
 
 #include "astring.h"
@@ -420,7 +421,7 @@ bool is_safe_filename(const char *name)
 ***************************************************************/
 bool is_ascii_name(const char *name)
 {
-  const char illegal_chars[] = {'|', '%', '"', ',', '*', '<', '>', '\'', '\0'};
+  const char illegal_chars[] = {'|', '%', '"', ',', '*', '<', '>', '\0'};
   int i, j;
 
   /* must not be NULL or empty */
@@ -775,12 +776,36 @@ char *user_home_dir(void)
       home_dir = mystrdup(env);	        /* never free()d */
       freelog(LOG_VERBOSE, "HOME is %s", home_dir);
     } else {
+
 #ifdef WIN32_NATIVE
-      home_dir=fc_malloc(PATH_MAX);
-      if (!getcwd(home_dir,PATH_MAX)) {
-	free(home_dir);
-	home_dir=NULL;
-	freelog(LOG_ERROR, "Could not find home directory (HOME is not set)");
+
+      /* some documentation at: 
+       * http://justcheckingonall.wordpress.com/2008/05/16/find-shell-folders-win32/
+       * http://archives.seul.org/or/cvs/Oct-2004/msg00082.html */
+
+      LPITEMIDLIST pidl;
+      LPMALLOC pMalloc;
+      
+      if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl))) {
+        
+        home_dir = fc_malloc(PATH_MAX);
+        
+        if (!SUCCEEDED(SHGetPathFromIDList(pidl, home_dir))) {
+          free(home_dir);
+          home_dir = NULL;
+          freelog(LOG_ERROR,
+            "Could not find home directory (SHGetPathFromIDList() failed)");
+        }
+        
+        SHGetMalloc(&pMalloc);
+        if (pMalloc) {
+          pMalloc->lpVtbl->Free(pMalloc, pidl);
+          pMalloc->lpVtbl->Release(pMalloc);
+        }
+
+      } else {
+        freelog(LOG_ERROR,
+          "Could not find home directory (SHGetSpecialFolderLocation() failed)");
       }
 #else
       freelog(LOG_ERROR, "Could not find home directory (HOME is not set)");
@@ -1260,6 +1285,9 @@ char *datafilename_required(const char *filename)
   }
 }
 
+/***************************************************************************
+  Language environmental variable.
+***************************************************************************/
 char *get_langname(void)
 {
   char *langname = NULL;      
@@ -1470,12 +1498,17 @@ enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
 			       size_t n_names,
 			       size_t max_len_name,
 			       m_pre_strncmp_fn_t cmp_fn,
+                               m_strlen_fn_t len_fn,
 			       const char *prefix,
 			       int *ind_result)
 {
   int i, len, nmatches;
 
-  len = strlen(prefix);
+  if (len_fn == NULL) {
+    len = strlen(prefix);
+  } else {
+    len = len_fn(prefix);
+  }
   if (len == 0) {
     return M_PRE_EMPTY;
   }
@@ -1655,9 +1688,15 @@ bool path_is_absolute(const char *filename)
     return FALSE;
   }
 
+#ifdef WIN32_NATIVE
+  if (strchr(filename, ':')) {
+    return TRUE;
+  }
+#else
   if (filename[0] == '/') {
     return TRUE;
   }
+#endif  
 
   return FALSE;
 }
